import React, { useState, useRef, useEffect, createRef, useCallback } from 'react';
import { Select } from '@highlight-ui/select';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import { useClickOutside, useForkRef } from '@highlight-ui/utils-hooks';
import classnames from 'classnames';
import { Tag } from '@highlight-ui/tag';
import { Typography } from '@highlight-ui/typography';
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var styles = {
  "heading-xsmall": "TagInput-module__ZJR5WSix__v4-0-30",
  "heading-small": "TagInput-module__dTMUzgJV__v4-0-30",
  "heading-medium": "TagInput-module__289H2vH5__v4-0-30",
  "heading-large": "TagInput-module__3wfOo_2-__v4-0-30",
  "heading-xlarge": "TagInput-module__KhVuGLEK__v4-0-30",
  "heading-2xl": "TagInput-module__oABf6DHP__v4-0-30",
  "body-small": "TagInput-module__2p4rtxMI__v4-0-30",
  "body-base": "TagInput-module__2JaIXXxs__v4-0-30",
  "inputElement": "TagInput-module__2lZcZwin__v4-0-30",
  "body-large": "TagInput-module__1Wh47hMI__v4-0-30",
  "body-strong-small": "TagInput-module__2dVvGjk7__v4-0-30",
  "body-strong-base": "TagInput-module__36-rep8a__v4-0-30",
  "body-strong-large": "TagInput-module__1v5Plnjc__v4-0-30",
  "page-heading": "TagInput-module__8Q1rf3vd__v4-0-30",
  "button": "TagInput-module__lwGT0vYH__v4-0-30",
  "form-label": "TagInput-module__2MPlvl9X__v4-0-30",
  "caption": "TagInput-module__3NpzTZSO__v4-0-30",
  "text-link-default": "TagInput-module__7Sf3UIHJ__v4-0-30",
  "text-link-hover": "TagInput-module__322lrNv-__v4-0-30",
  "table-data-regular": "TagInput-module__3CPb-PhF__v4-0-30",
  "table-data-bold": "TagInput-module__Hv119OtC__v4-0-30",
  "container": "TagInput-module__3Hiv8t1e__v4-0-30",
  "inputOutline": "TagInput-module__1SxgMouL__v4-0-30",
  "focused": "TagInput-module__16f4thar__v4-0-30",
  "tag": "TagInput-module__2Lqw0ZEv__v4-0-30",
  "isHidden": "TagInput-module__a9UWROgi__v4-0-30",
  "additionalTags": "TagInput-module__1VcTcx6n__v4-0-30",
  "suffix": "TagInput-module__2YnRD65a__v4-0-30"
};

var getTagsShownNumber = function (tagElements, containerElement, suffixRef) {
  if (!tagElements.length) return 0;
  if (!containerElement.current) return 0;
  var prefixWidth = suffixRef.current ? suffixRef.current.offsetWidth + 24 : 0;
  var maxWidth = containerElement.current.offsetWidth - 40 - prefixWidth;
  var value = tagElements.slice(1).reduce(function (acc, item) {
    if (!item.current) return acc;

    if (maxWidth > acc.totalWidth + item.current.offsetWidth + 4) {
      return {
        items: acc.items + 1,
        totalWidth: acc.totalWidth + item.current.offsetWidth + 4
      };
    }

    return {
      items: acc.items,
      totalWidth: acc.totalWidth + item.current.offsetWidth
    };
  }, {
    items: 1,
    totalWidth: tagElements[0].current ? tagElements[0].current.offsetWidth + 4 : 0
  });
  return value.items;
};

var TagInput = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b;

  var tags = _a.tags,
      onInputClick = _a.onInputClick,
      onInputChange = _a.onInputChange,
      onTagAdd = _a.onTagAdd,
      onTagRemove = _a.onTagRemove,
      suffix = _a.suffix,
      className = _a.className,
      metadata = _a.metadata,
      selectRef = _a.selectRef;

  var _c = useState(''),
      inputValue = _c[0],
      setInputValue = _c[1];

  var _d = useState(false),
      isFocused = _d[0],
      setFocus = _d[1];

  var _e = useState([]),
      elRefs = _e[0],
      setElRefs = _e[1];

  var inputRef = useRef(null);
  var innerRef = useRef(null);
  var suffixRef = useRef(null);

  var clearInput = function () {
    setInputValue('');

    if (onInputChange) {
      onInputChange('');
    }
  }; // TODO: refactor and remove selectRef from TagInput [PPUI-2030]


  var clickOutsideRef = useClickOutside(function (event) {
    var _a;

    setFocus(false);

    if (inputValue) {
      if (!selectRef || selectRef && !((_a = selectRef.current) === null || _a === void 0 ? void 0 : _a.contains(event.target))) {
        onTagAdd(inputValue);
      }

      clearInput();
    }
  });
  var containerRef = useForkRef([ref, clickOutsideRef, innerRef]);

  var _f = useState(0),
      tagsShownNumber = _f[0],
      setTagsShownNumber = _f[1];

  useEffect(function () {
    setElRefs(function (prevState) {
      return Array(tags.length).fill(0).map(function (_, i) {
        return prevState[i] || /*#__PURE__*/createRef();
      });
    });
  }, [tags]);
  useEffect(function () {
    if (elRefs && innerRef) {
      setTagsShownNumber(getTagsShownNumber(elRefs, innerRef, suffixRef));
    }
  }, [elRefs, innerRef]);
  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(styles.container, className)
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("div", {
    role: "none",
    onClick: function (e) {
      var _a;

      e.stopPropagation();
      e.preventDefault();
      (_a = inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    },
    ref: containerRef,
    className: classnames(styles.inputOutline, (_b = {}, _b[styles.focused] = isFocused, _b))
  }, tags.map(function (tag, index) {
    var _a;

    return (
      /*#__PURE__*/
      // eslint-disable-next-line react/no-array-index-key
      React.createElement(React.Fragment, {
        key: index
      }, /*#__PURE__*/React.createElement(Tag, __assign({}, tag, {
        metadata: suffixMetadata(metadata, "tag-" + index),
        ref: elRefs[index],
        className: classnames(styles.tag, (_a = {}, _a[styles.isHidden] = !isFocused && index >= tagsShownNumber, _a)),
        onRemove: function (e) {
          e.stopPropagation();
          onTagRemove(tag, index);
        }
      })), !isFocused && index === tagsShownNumber - 1 && tags.length > tagsShownNumber && /*#__PURE__*/React.createElement(Tag, {
        className: styles.additionalTags,
        text: "+" + (tags.length - tagsShownNumber)
      }))
    );
  }), /*#__PURE__*/React.createElement("input", {
    ref: inputRef,
    className: styles.inputElement,
    value: inputValue,
    onKeyDown: function (event) {
      var value = event.currentTarget.value;

      if (value) {
        if (event.key === 'Enter' || event.key === 'Tab' || event.key === ',') {
          onTagAdd(value);
          clearInput();
        }
      }
    },
    onChange: function (event) {
      var input = event.target.value;

      if (input !== ',') {
        setInputValue(input);

        if (onInputChange) {
          onInputChange(input);
        }
      }
    },
    onFocus: function () {
      if (!isFocused) {
        setFocus(true);

        if (onInputClick) {
          onInputClick();
        }
      }
    }
  }), suffix && /*#__PURE__*/React.createElement(Typography, {
    ref: suffixRef,
    token: "body-strong-small",
    className: styles.suffix
  }, suffix)));
});

function TagPickerWithRef(_a, ref) {
  var tags = _a.tags,
      onInputChange = _a.onInputChange,
      onTagAdd = _a.onTagAdd,
      onTagRemove = _a.onTagRemove,
      options = _a.options,
      className = _a.className,
      metadata = _a.metadata,
      suffix = _a.suffix;

  var _b = useState(''),
      highlightedText = _b[0],
      setHighlightedText = _b[1];

  var handleOnInputChange = useCallback(function (input) {
    setHighlightedText(input);

    if (onInputChange) {
      onInputChange(input);
    }
  }, [onInputChange]);
  var renderTriggerCallback = useCallback(function (_a) {
    var onClick = _a.onClick,
        triggerRef = _a.ref,
        selectRef = _a.selectRef;
    return /*#__PURE__*/React.createElement(TagInput, {
      metadata: suffixMetadata(metadata, 'tag-input'),
      tags: tags,
      onInputClick: onClick,
      onInputChange: handleOnInputChange,
      onTagAdd: onTagAdd,
      onTagRemove: onTagRemove,
      suffix: suffix,
      ref: triggerRef,
      selectRef: selectRef
    });
  }, [handleOnInputChange, metadata, onTagAdd, onTagRemove, suffix, tags]);
  return /*#__PURE__*/React.createElement(Select, {
    ref: ref,
    className: className,
    metadata: metadata,
    renderTrigger: renderTriggerCallback,
    closeOnSelect: true,
    highlightedText: highlightedText,
    searchable: false,
    options: options,
    renderEmptyState: function () {
      return null;
    },
    onSelect: function (option) {
      return onTagAdd(option[0].label);
    }
  });
}

TagPickerWithRef.displayName = 'TagPicker';
var TagPicker = /*#__PURE__*/React.forwardRef(TagPickerWithRef);
export { TagInput, TagPicker };
//# sourceMappingURL=index.js.map
